# USB/UART serial boot loader for the !final! FPGA image of RAM.hex
# Once this is burned, there is no further need to recompile/burn a new RAM.hex into the FPGA.
# The focus shifts to quick-turnaround Tigerasm.py chops.
# A decent serial terminal emulator with a reliable unfucked binary file send is required.

# To avoid clobbering the RAM used by the serial loader, the first line of app source must be something like:
#   include('FPGAburn\serload.asm')
# Thus, app_start is defined.
# Also, the last line of app source must be exactly:
#lbl('app_end')
# Such apps load and start execution at app_start, the lowest safe address.

# After assembly, apps are saved from app_start to app_end-1 via the Tigerasm.py save_app('yourapp.app') procedure.
# The output file starts with a 2-byte representation of the app_end address,
# followed by 2-byte representations of each 12-bit cell between app_start and app_end.

  # start execution at RAM[0] after reset
  # sign on
  lit(fwd('signon_msg')) # -> addr
lbl('signon')
    tt(); ld(); lit(fwd('signon_done'))
    jz() # suppress trailing 0
lbl('signon_char') # -> addr
        lit(tx_status); ld(); lit(signon_char) # await tx ready
      jz(); tt(); ld(); # send next character
      lit(uart); st(); inc(); # -> addr+1
      lit(signon); jmp()
lbl('signon_done')
  # set load start address
  lit(fwd('app_start')); lit(fwd('serload_dest')); st()
  # get app_end, flood the stack
  lit(fwd('rx2x6')); call() # -> app_end
  tt(); tt() # -> app_end app_end app_end
lbl('serload_payload')
    lit(fwd('serload_dest')); ld(); bxor()
    lit(fwd('serload_done')); jz(); lit(fwd('rx2x6')) # if app_end not reached, get another pair
      call() # -> app_end app_end app_end d
      lit(fwd('serload_dest')); ld(); st()
      lit(fwd('serload_dest')); ld(); inc() # bump dest addr
      lit(fwd('serload_dest')); st(); lit(serload_payload)
      jmp()
lbl('serload_done')
  lit(fwd('app_start')); jmp() # execute the downloaded app

lbl('signon_msg'); ascii('Tiger is ready to load.\r\n')
lbl('serload_dest'); data([0])

#
# receive a pair of 6-bit data bytes, ls byte first, pack
# x x y z -> z z z d
#
lbl('rx2x6')
    # get ls byte
    lit(rx_status); ld(); lit(rx2x6)
    jz(); lit(uart); ld();
lbl('rx2x61')
    # get ms byte
    lit(rx_status); ld(); lit(rx2x61)
    jz(); lit(uart); ld();
  # shift left 6
  tt(); add(); tt()
  add(); tt(); add()
  tt(); add(); tt()
  add(); tt(); add()
  # merge
  add(); ret()    

lbl('app_start') # must be on the last line of serload.asm