// Tiger SOC

module TigerSOC (
  input clk_i,
  input rst_i,
  input rxd_i, // USB/UART
  input [11:0] pin_i, // parallel inputs
  output call_overflow_o, // too many call levels, halt and light a red LED
  output ret_underflow_o, // too many return levels, halt and light a red LED
  output txd_o, // USB/UART
  output [11:0] pout_o // parallel outputs
);

reg rst; // external reset synchronizer
always @(posedge clk_i) rst <= rst_i;

wire storex, loadex;
wire [11:0] address, read_data, write_data;

TigerCPU u0 (
  .clk_i(clk_i),
  .rst_i(rst),
  .dat_i(read_data),
  .call_overflow_o(call_overflow_o),
  .ret_underflow_o(ret_underflow_o),
  .storex_o(storex),
  .loadex_o(loadex),
  .adr_o(address),
  .dat_o(write_data)
);

wire [11:0] RAM_data;

RAM u1 (
  .clk_i(clk_i),
  .we_i(storex),
  .adr_i(address),
  .dat_i(write_data),
  .dat_o(RAM_data)
);

wire pio_sel, timer_high_sel, timer_low_sel, rx_ready_sel, tx_ready_sel, uart_sel;

iodecode u2 (
  .adr_i(address),
  .pio_o(pio_sel),
  .timer_high_o(timer_high_sel),
  .timer_low_o(timer_low_sel),
  .rx_status_o(rx_ready_sel),
  .tx_status_o(tx_ready_sel),
  .uart_o(uart_sel)
);

wire pio_we = pio_sel & storex;

pout u3 (
  .clk_i(clk_i),
  .rst_i(rst),
  .we_i(pio_we),
  .dat_i(write_data),
  .dat_o(pout_o)
);

wire [23:0] timer;

timer u4 (
  .clk_i(clk_i),
  .rst_i(rst),
  .dat_o(timer)
);

wire read_arx = uart_sel & loadex;
wire rx_ready;
wire [7:0] rx_byte;

arx u5 (
  .clk_i(clk_i),
  .rst_i(rst),
  .rxd_i(rxd_i),
  .rst_ready_i(read_arx),
  .ready_o(rx_ready),
  .dat_o(rx_byte)
);

wire write_atx = uart_sel & storex;
wire tx_ready;

atx u6 (
  .clk_i(clk_i),
  .rst_i(rst),
  .we_i(write_atx),
  .dat_i(write_data[7:0]),
  .txd_o(txd_o),
  .ready_o(tx_ready)
);

readmux u7 (
  .pio_i(pio_sel),
  .pin_i(pin_i),
  .timer_high_i(timer_high_sel),
  .timer_low_i(timer_low_sel),
  .timer_i(timer),
  .rx_status_i(rx_ready_sel),
  .rx_ready_i(rx_ready),
  .tx_status_i(tx_ready_sel),
  .tx_ready_i(tx_ready),
  .uart_i(uart_sel),
  .rx_byte_i(rx_byte),
  .RAM_dat_i(RAM_data),
  .dat_o(read_data)
);

endmodule