include('FPGAburn/serload.asm') # must be on the first line

#  lit(app_start); call() # test call overflow
#  ret() # test return underflow

# play with a 24-bit digital controlled oscillator
# a squarewave output is available on Pmod-4, Pmod-5 is GND
# one may also generate a sawtooth output by connecting msbs of pout_o to a parallel DAC
# see TigerSOC.pcf

  lit(fwd('init')); jmp() # avoid some of fwd() hell

lbl('phacc0'); data([0]) # ls phase accumulator
lbl('phacc1'); data([0]) # ms phase accumulator
lbl('phinc0'); data([0]) # ls phase increment
lbl('phinc1'); data([0]) # ms phase increment

lbl('emit') # x y z ascii -> z z z z
    lit(tx_status); ld(); lit(emit); jz() # await tx ready
  lit(uart); st(); ret()

#
# command and its stooges
# any of these may trash the stack
# 
lbl('got_plus') # increment phinc
  lit(phinc0); ld(); lit(1); adx() # -> phinc0+1 carry
  lit(phinc1); ld(); add() # -> phinc0+1 phinc1+carry
  lit(phinc1); st(); lit(phinc0); st(); ret()

lbl('got_minus') # decrement phinc
  lit(phinc0); ld(); lit(ALLBITS); adx(); lit(fwd('got_minus_borrow')); jz() # -> phinc0-1
    lit(phinc0); st(); ret()
lbl('got_minus_borrow')
  lit(phinc0); st();
  lit(phinc1); ld(); lit(ALLBITS); add(); lit(phinc1); st(); ret()

lbl('got_upoctave') # double phinc
  lit(phinc0); ld(); tt(); adx() # -> phinc0<<1 carry
  lit(phinc1); ld(); tt(); add(); add() # -> phinc0<<1 phinc1<<1+carry
  lit(phinc1); st(); lit(phinc0); st(); ret()

lbl('got_downoctave') # halve phinc
  lit(phinc1); ld(); lit(1); band(); lit(fwd('got_downoctave_even')); jz()
    # odd phinc1
    lit(phinc0); ld(); asr(); lit(MAXPOS); band(); lit(SIGNBIT); bxor(); lit(phinc0); st()
    lit(phinc1); ld(); asr(); lit(phinc1); st(); ret()
lbl('got_downoctave_even')
  lit(phinc0); ld(); asr(); lit(MAXPOS); band(); lit(phinc0); st()
  lit(phinc1); ld(); asr(); lit(phinc1); st(); ret()

lbl('got_A440') # set phinc to generate 440 Hz
  lit(20300 & ALLBITS); lit(phinc0); st()
  lit(20300 >> 12); lit(phinc1); st(); ret()

lbl('command') # respond to USB/UART keystrokes, -> trash
  lit(rx_status); ld(); lit(fwd('no_command')); jz()
    lit(uart); ld(); tt(); tt() # -> keystroke keystroke keystroke
    lit(emit); call()
    # resolve defined keystroke commands
    lit(ord('-')); bxor(); lit(got_minus); jz()
    lit(ord('+')); bxor(); lit(got_plus); jz()
    lit(ord('<')); bxor(); lit(got_upoctave); jz()
    lit(ord('>')); bxor(); lit(got_downoctave); jz()
    lit(ord('a')); bxor(); lit(got_A440); jz()
lbl('no_command')
  ret()
#
# end command and its stooges
#

lbl('init')
  lit(got_A440); call() # set default Hz
lbl('doit') # main loop troika'd for timing analysis, 33 clks
    lit(command); call() # nominal 8 clks, user control of phinc
    lit(phacc0); ld(); lit(phinc0) # 4 clks, -> phacc0 @phinc0
    ld(); adx(); lit(phacc1); # 4 clks, -> phacc0' carry @phacc1
    ld(); add(); lit(phinc1); # 4 clks, -> phacc0' phacc1+carry @phinc1
    ld(); add(); tt() # 3 clks, -> phacc0' phacc1' phacc1'
    lit(pio); st(); lit(phacc1) # 4 clks, -> phacc0' phacc1' @phacc1
    st(); lit(phacc0); st() # 4 clks
    lit(doit); jmp() # 2 clks
# fs = fclk/33
# fout = fs*phinc/2**24
# kf   = 0.021674416281960228 # Hz, phinc = 1
# The magic number 20300 in got_A440 was calculated 440/kf.

lbl('app_end') # must be on the last line