/*
asynchronous transmitter, 8 data bits, no parity, 1 stop bit
the bit rate is determined below
___     ___ ___ ___ ___ ___ ___ ___ _______ ___
   \___X___X___X___X___X___X___X___X___/   X___
   start 0   1   2   3   4   5   6   7 stop
*/

module atx (
  input clk_i,
  input rst_i,
  input we_i,        // write tx char, reset ready_o
  input [7:0] dat_i, // tx char
  output txd_o,      // to the USB/UART converter
  output ready_o     // transmitter ready
);
// direct output
reg ready_o;

// lsb-first parallel in serial out
reg [8:0] piso; // 9 bits
assign txd_o = piso[0]; // current tx bit

// bit rate generator
reg [10:0] brg;
wire next_bit = brg == 1249; // (12000000/9600)-1
always @(posedge clk_i)
  if (rst_i | ready_o | next_bit) brg <= 0;
  else brg <= brg + 1;

// transmitted bits counter
reg [3:0] txbits;

// the transmitter
always @(posedge clk_i)
  if (rst_i) begin
    ready_o <= 1;
    piso[0] <= 1;
    txbits <= 0;
  end else if (ready_o) begin
    if (we_i) begin // accept a new char
      piso <= {dat_i, 1'b0}; // prepend start bit
      ready_o <= 0;
    end
  end else begin // transmit pending bits
    if (txbits == 10) begin // stop bit was transmitted
      txbits <= 0;
      ready_o <= 1;
    end else if (next_bit) begin // time to shift a pending bit
      piso <= {1'b1, piso[8:1]}; // the 10th bit will be a stop bit
      txbits <= txbits + 1;
    end
  end

endmodule
