# Tiger subroutine library

#
# binary or
# z z d1 d2 -> z z z d1|d2
#
lbl('bor_d2'); data([0])
lbl('bor')
  lit(bor_d2); st(); tt()
  inv(); lit(bor_d2); ld()
  band(); bxor(); ret()

#
# shlN family
# y y z d -> y y z d<<N
#
lbl('shl4'); tt(); add()
lbl('shl3'); tt(); add()
lbl('shl2'); tt(); add()
lbl('shl1'); tt(); add(); ret()

#
# test d1==d2
# z z d1 d2 -> z z z flag
#
lbl('eq')
  bxor()
#
# test d==0
# y y z d -> y y z flag
#
lbl('eq0')
  lit(fwd('eq01')); jz(); lit(0)
    ret()
lbl('eq01')
  lit(ALLBITS); ret()

#
# test d1<d2
# z z d1 d2 -> z z z flag
#
lbl('lt')
  inv(); inc(); add() # subtract
#
# test d<0
# y y z d -> y y z flag
#
lbl('lt0')
  lit(SIGNBIT); band(); ret()

#
# minmax, factor of min and max returns flag, copies d2 and d1
# z z d1 d2 -> z z z flag:d1<d2
#
lbl('minmax_d2'); data([0])
lbl('minmax_d1'); data([0])
lbl('minmax')
  lit(minmax_d2); st(); tt()
  lit(minmax_d1); st(); lit(minmax_d2)
  ld(); lit(lt); jmp() # eliminate tail call

#
# minimum
# z z d1 d2 -> z z z d
#
lbl('min')
  lit(minmax); call()
  lit(fwd('min1')); jz(); lit(minmax_d1)
    ld(); ret()
lbl('min1')
  lit(minmax_d2); ld(); ret()

#
# maximum
# z z d1 d2 -> z z z d
#
lbl('max')
  lit(minmax); call()
  lit(fwd('max1')); jz(); lit(minmax_d2)
    ld(); ret()
lbl('max1')
  lit(minmax_d1); ld(); ret()

#
# absolute value
# y y z d -> y y z +d
# invalid result if d==SIGNBIT
#
lbl('abs_d'); data([0])
lbl('abs')
  lit(abs_d); st(); lit(abs_d)
  ld(); lit(lt0); call()
  lit(fwd('abs1')); jz(); lit(abs_d)
    ld(); inv(); inc()
    ret()
lbl('abs1')
  lit(abs_d); ld(); ret()

#
# sine
# y y z phase -> y y z amplitude
# phase is unsigned, analogous to 0..2pi
#
lbl('sine_lut'); data([int(MAXPOS*math.sin(n*math.tau/256)) for n in range(256)])
lbl('sine')
  asr(); asr(); asr()
  asr(); lit(255); band()
  lit(sine_lut); add(); ld()
  ret()

#
# unsigned 4x4 multiply
# z z d1 d2 -> z z z d1*d2
# d1 and d2 must both be 0..15
#
lbl('umul4x4_lut'); data([a*b for a in range(16) for b in range(16)])
lbl('umul4x4')
  lit(shl4); call()
  bxor(); lit(umul4x4_lut); add()
  ld(); ret()

#
# copy memory block
# count count src-addr dest-addr -> count count count 0
# count must be within 0..SIGNBIT
#
lbl('copy_src'); data([0])
lbl('copy_dest'); data([0])
lbl('copy')
  lit(copy_dest); st(); lit(copy_src)
  st(); inv(); inc() # -> count count count -count
lbl('copy1')
    tt(); lit(fwd('copyx')); jz()
      lit(copy_src); ld(); ld()
      lit(copy_dest); ld(); st()
      lit(copy_src); ld(); inc()
      lit(copy_src); st(); lit(copy_dest)
      ld(); inc(); lit(copy_dest)
      st(); inc(); lit(copy1)
      jmp()
lbl('copyx')
  ret()

#
# translate a hex digit to a lowercase ASCII hex character
# y y z d -> y y z ascii
# the lowest 4 bits of d are significant
#
lbl('hex_to_ascii_lut'); ascii('0123456789abcdef')
lbl('hex_to_ascii')
  lit(15); band(); lit(hex_to_ascii_lut)
  add(); ld(); ret

#
# translate a lowercase ASCII hex character to a hex digit
# y y z ascii -> z z z d
# d will be 16 if ascii is not a lowercase hex character
#
lbl('ascii_to_hex_ascii'); data([0])
lbl('ascii_to_hex')
  lit(ascii_to_hex_ascii); st(); lit(0) # -> y y z 0 # search index
lbl('ascii_to_hex1')
    tt(); lit(hex_to_ascii_lut); add()
    ld(); lit(ascii_to_hex_ascii); ld()
    bxor(); lit(fwd('ascii_to_hexx')); jz()
      inc(); tt(); lit(16)
      bxor(); lit(fwd('ascii_to_hexx')); jz()
        lit(ascii_to_hex1); jmp()
lbl('ascii_to_hexx')
  ret